import torch
from sentence_transformers import SentenceTransformer
from FlagEmbedding import BGEM3FlagModel

class BGEM3Encoder():
    def __init__(self, model_path = 'BAAI/bge-m3', batch_size = 128, max_length = 512):
        self.model = BGEM3FlagModel(model_path, use_fp16=True) # Setting use_fp16 to True speeds up computation with a slight performance degradation
        self.batch_size = batch_size
        self.max_length = max_length

    def encode(self, texts):
        if isinstance(texts, str):
            texts = [texts]
        elif isinstance(texts, list):
            pass
        else:
            raise ValueError("Only support string or list[string]")
        output = self.model.encode(texts,
                              max_length=self.max_length,
                              return_dense=False,
                              return_sparse=True,
                              return_colbert_vecs=False,
                              batch_size=self.batch_size)
        sparse_data = output['lexical_weights']
        sorted_sparse_data = []
        for sparse in sparse_data:
            sparse_sorted = sorted(sparse.items(), key=lambda item: int(item[0]), reverse=False)
            sparse_indices = [x[0] for x in sparse_sorted]
            sparse_values = [x[1] for x in sparse_sorted]
            sorted_sparse_data.append({
                "indices": sparse_indices,
                "values": sparse_values,
            })

        return sorted_sparse_data
